@@ -1,10 +1,9 @@
 package Net::DNS::Resolver::MSWin32;
 
-#
-# $Id: MSWin32.pm 1345 2015-05-19 10:07:43Z willem $
-#
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1345 $)[1];
+use strict;
+use warnings;
+our $VERSION = (qw$Id: MSWin32.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+
 
 =head1 NAME
 
@@ -13,52 +12,37 @@ Net::DNS::Resolver::MSWin32 - MS Windows resolver class
 =cut
 
 
-use strict;
 use base qw(Net::DNS::Resolver::Base);
-
 use Carp;
 
-BEGIN {
-	use vars qw($Registry);
+use constant WINHLP => defined eval 'require Win32::IPHelper';	## no critic
+use constant WINREG => defined eval 'use Win32::TieRegistry qw(KEY_READ REG_DWORD); 1';	## no critic
 
-	use constant WINHLP => eval {	## use Win32::Helper;	# hidden from static analyser
-		require Win32::IPHelper;
-	} || 0;
-
-	Win32::IPHelper->import if WINHLP;
-
-	use constant WINREG => eval {	## use Win32::TieRegistry;
-		require Win32::TieRegistry;
-	} || 0;
-
-	Win32::TieRegistry->import(qw(KEY_READ REG_DWORD)) if WINREG;
-}
+our $Registry;
 
 
-sub _untaint { map defined && /^(.+)$/ ? $1 : (), @_; }
-
-
-sub init {
-	my $defaults = shift->defaults;
+sub _init {
+	my $defaults = shift->_defaults;
 
 	my $debug = 0;
 
 	my $FIXED_INFO = {};
 
-	if ( my $ret = Win32::IPHelper::GetNetworkParams($FIXED_INFO) ) {
-		Carp::croak "GetNetworkParams() error %u: %s\n", $ret, Win32::FormatMessage($ret);
-	} elsif ($debug) {
+	my $err = Win32::IPHelper::GetNetworkParams($FIXED_INFO);
+	croak "GetNetworkParams() error %u: %s\n", $err, Win32::FormatMessage($err) if $err;
+
+	if ($debug) {
 		require Data::Dumper;
 		print Data::Dumper::Dumper $FIXED_INFO;
 	}
 
 
 	my @nameservers = map { $_->{IpAddress} } @{$FIXED_INFO->{DnsServersList}};
-	$defaults->nameservers( _untaint @nameservers );
+	$defaults->nameservers(@nameservers);
 
 	my $devolution = 0;
-	my @searchlist = _untaint lc $FIXED_INFO->{DomainName};
-	$defaults->domain(@searchlist);
+	my $domainname = $FIXED_INFO->{DomainName} || '';
+	my @searchlist = grep {length} $domainname;
 
 	if (WINREG) {
 
@@ -76,7 +60,7 @@ sub init {
 		}
 
 		if ( defined $reg_tcpip ) {
-			my $searchlist = lc $reg_tcpip->GetValue('SearchList') || '';
+			my $searchlist = $reg_tcpip->GetValue('SearchList') || '';
 			push @searchlist, split m/[\s,]+/, $searchlist;
 
 			my ( $value, $type ) = $reg_tcpip->GetValue('UseDomainNameDevolution');
@@ -86,23 +70,26 @@ sub init {
 
 
 	# fix devolution if configured, and simultaneously
-	# make sure no dups (but keep the order)
+	# eliminate duplicate entries (but keep the order)
 	my @list;
 	my %seen;
-	foreach my $entry (@searchlist) {
-		push( @list, $entry ) unless $seen{$entry}++;
+	foreach (@searchlist) {
+		s/\.+$//;
+		push( @list, $_ ) unless $seen{lc $_}++;
 
 		next unless $devolution;
 
-		# as long there are more than two pieces, cut
-		while ( $entry =~ m#\..+\.# ) {
-			$entry =~ s#^[^\.]+\.(.+)$#$1#;
-			push( @list, $entry ) unless $seen{$entry}++;
+		# while there are more than two labels, cut
+		while (s#^[^.]+\.(.+\..+)$#$1#) {
+			push( @list, $_ ) unless $seen{lc $_}++;
 		}
 	}
-	$defaults->searchlist( _untaint @list );
+	$defaults->searchlist(@list);
 
-	$defaults->read_env;
+	%$defaults = Net::DNS::Resolver::Base::_untaint(%$defaults);
+
+	$defaults->_read_env;
+	return;
 }
 
 
